﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Analysis;
using HEAL.Attic;

namespace HeuristicLab.DatastreamAnalysis {
  /// <summary>
  /// A row of data ranges.
  /// </summary>
  [Item("RangeDataRow", "A DataRow with double tuples")]
  [StorableType("38367CFE-AEE7-44C2-BF10-CB42F25BB533")]
  public class RangeDataRow : NamedItem {
    private DataRowVisualProperties visualProperties;
    public DataRowVisualProperties VisualProperties {
      get { return visualProperties; }
      set {
        if (visualProperties != value) {
          if (value == null) throw new ArgumentNullException("VisualProperties");
          if (visualProperties != null) visualProperties.PropertyChanged -= new PropertyChangedEventHandler(VisualProperties_PropertyChanged);
          visualProperties = value;
          visualProperties.PropertyChanged += new PropertyChangedEventHandler(VisualProperties_PropertyChanged);
          OnVisualPropertiesChanged();
        }
      }
    }
    private ObservableList<Tuple<double,double>> values;
    public ObservableList<Tuple<double, double>> Values {
      get { return values; }
    }

    #region Persistence Properties
    [Storable(Name = "VisualProperties")]
    private DataRowVisualProperties StorableVisualProperties {
      get { return visualProperties; }
      set {
        visualProperties = value;
        visualProperties.PropertyChanged += new PropertyChangedEventHandler(VisualProperties_PropertyChanged);
      }
    }
    [Storable(Name ="values")]
    private IEnumerable<Tuple<double, double>> StorableValues {
      get { return values; }
      set { values = new ObservableList<Tuple<double, double>>(value); }
    }
    #endregion

    [StorableConstructor]
    protected RangeDataRow(StorableConstructorFlag _) : base(_) { }
    protected RangeDataRow(RangeDataRow original, Cloner cloner)
      : base(original, cloner) {
      this.VisualProperties = (DataRowVisualProperties)cloner.Clone(original.visualProperties);
      this.values = new ObservableList<Tuple<double, double>>(original.values);
    }
    public RangeDataRow() : this("RangeDataRow") { }
    public RangeDataRow(string name)
      : base(name) {
      if (string.IsNullOrEmpty(name)) throw new ArgumentException("Name of a RangeDataRow cannot be empty", name);
      VisualProperties = new DataRowVisualProperties(name);
      values = new ObservableList<Tuple<double,double>>();
    }
    public RangeDataRow(string name, string description)
      : base(name, description) {
      if (string.IsNullOrEmpty(name)) throw new ArgumentException("Name of a RangeDataRow cannot be empty", name);
      VisualProperties = new DataRowVisualProperties(name);
      values = new ObservableList<Tuple<double, double>>();
    }
    public RangeDataRow(string name, string description, IEnumerable<Tuple<double, double>> values)
      : base(name, description) {
      if (string.IsNullOrEmpty(name)) throw new ArgumentException("Name of a RangeDataRow cannot be empty", name);
      VisualProperties = new DataRowVisualProperties(name);
      this.values = new ObservableList<Tuple<double, double>>(values);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RangeDataRow(this, cloner);
    }

    public event EventHandler VisualPropertiesChanged;
    protected virtual void OnVisualPropertiesChanged() {
      EventHandler handler = VisualPropertiesChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    private void VisualProperties_PropertyChanged(object sender, PropertyChangedEventArgs e) {
      OnVisualPropertiesChanged();
    }
    protected override void OnNameChanged() {
      base.OnNameChanged();
      VisualProperties.DisplayName = Name;
    }
  }
}
